/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.launcher

import org.apache.camel.model.ChoiceDefinition
import org.apache.camel.model.ProcessorDefinition
import org.apache.camel.language.groovy.CamelGroovyMethods
import org.apache.camel.builder.ExpressionBuilder
import org.apache.camel.Expression

import metridoc.impl.iterator.LineIteratorProvider
import metridoc.processor.ClosureProcessor
import metridoc.aggregator.InflightAggregationWrapper
import metridoc.aggregator.BodyAggregator
import metridoc.polling.OnePollOnlyStrategy
import metridoc.processor.SimpleAggregatorProcessor
import metridoc.predicate.IdempotentPredicate

ProcessorDefinition.metaClass.filter = { filter ->
    if (filter instanceof Closure) {
        filter = CamelGroovyMethods.toExpression(filter)
    }
    delegate.filter(filter);
}

ChoiceDefinition.metaClass.when = { filter ->
    if (filter instanceof Closure) {
        filter = CamelGroovyMethods.toExpression(filter)
    }
    delegate.when(filter);
}

ProcessorDefinition.metaClass.process = {filter ->
    if (filter instanceof Closure) {
        filter = new ClosureProcessor(filter)
    }
    delegate.process(filter);
}

ProcessorDefinition.metaClass.aggregateBody = {
    def expression = ExpressionBuilder.constantExpression(true)
    delegate.aggregate(expression, new InflightAggregationWrapper(new BodyAggregator())).
                    completionSize(500).completionTimeout(500);
}

ProcessorDefinition.metaClass.aggregateBody = {int size, long timeout=500 ->
    def expression = ExpressionBuilder.constantExpression(true)
    delegate.aggregate(expression, new InflightAggregationWrapper(new BodyAggregator())).
                    completionSize(size).completionTimeout(timeout);
}

ProcessorDefinition.metaClass.splitByLine = {
    Expression bean = ExpressionBuilder.beanExpression(LineIteratorProvider.class, "create");
    delegate.split(bean).streaming()
}

ProcessorDefinition.metaClass.splitByLine = {int threads, int maxPoolSize = 20, int maxQueueSize = 20 ->
    Expression bean = ExpressionBuilder.beanExpression(LineIteratorProvider.class, "create");
    delegate.split(bean).streaming().threads(threads, maxPoolSize).maxQueueSize(maxQueueSize)
}

onePollOnly = new OnePollOnlyStrategy()